import uuid
from pymongo.errors import BulkWriteError

class RubricaRepository:
    def __init__(self, db):
        self.__db = db

    async def inserir_rubrica(self, rubrica: dict) -> bool:
        
        try:
            rubricas_no_db = await self.__db.rubricas.find_one(
                {"solicitacaoId": rubrica["solicitacaoId"]}
            )

            rubrica['id'] = str(uuid.uuid4())

            if rubricas_no_db is None:
                await self.__db.rubricas.insert_one(rubrica)
                return True

            await self.__db.rubricas.delete_one(
                   {"solicitacaoId": rubrica["solicitacaoId"]}
            )
            await self.__db.rubricas.insert_one(rubrica)
            return True
        except Exception as e:
            print(f"❌ Erro ao inserir o rubrica: {e}")
            return False
        
    async def buscar_por_solicitacao_id(self, solicitacaoId: int) -> dict:
        try:
            return await self.__db.rubricas.find_one({"solicitacaoId": solicitacaoId})
        except Exception as e:
            print(f"❌ Erro ao buscar rubricas por solicitacaoId: {e}")
            return {}


from datetime import datetime
import uuid
from pymongo.errors import BulkWriteError

from libretificacaotjcore.dtos.processo_dto import ProcessoDto
from libretificacaotjcore.enums.e_fase_retificacao import EFaseRetificacao

class TempoProcessoRepository:
    def __init__(self, db):
        self.__db = db

    async def inserir_processo(self, *, processo: ProcessoDto) -> bool:
        try:
            processo_dict = processo.model_dump(exclude_none=True)
            processo_dict["DataInicio"] = processo.DataInicio.strftime("%Y-%m-%d")
            processo_dict["DataFim"] = processo.DataFim.strftime("%Y-%m-%d")
            processo_dict["FaseDescricao"] = EFaseRetificacao(processo.Fase).name

            processo_no_db = await self.__db.tempo_processos.find_one(
                {
                    "SolicitacaoId": processo.SolicitacaoId,
                    "Fase": processo.Fase,
                    "DataInicio": processo_dict["DataInicio"],
                    "DataFim": processo_dict["DataFim"],
                }
            )
            
            processo_dict["id"] = str(uuid.uuid4())

            if processo_no_db is None:
                processo_dict["InicioProcesso"] = datetime.now()
                await self.__db.tempo_processos.insert_one(processo_dict)
                return True

            await self.__db.tempo_processos.delete_one(
                {
                    "SolicitacaoId": processo.SolicitacaoId,
                    "Fase": processo.Fase,
                    "DataInicio": processo_dict["DataInicio"],
                    "DataFim": processo_dict["DataFim"],
                }
            )

            processo_dict["InicioProcesso"] = datetime.now()
            await self.__db.tempo_processos.insert_one(processo_dict)
            return True
        
        except Exception as e:
            print(f"❌ Erro ao inserir o processo: {e}")
            return False
        
    async def atualizar_processo(self, *, processo: ProcessoDto) -> bool:
        try:
            processo_dict = processo.model_dump()
            processo_dict["DataInicio"] = processo.DataInicio.strftime("%Y-%m-%d")
            processo_dict["DataFim"] = processo.DataFim.strftime("%Y-%m-%d")
             
            processo_no_db = await self.__db.tempo_processos.find_one(
                {
                    "SolicitacaoId": processo.SolicitacaoId,
                    "Fase": processo.Fase,
                    "DataInicio": processo_dict["DataInicio"],
                    "DataFim": processo_dict["DataFim"],
                }
            )

            if processo_no_db is None:
                return False
            
            processo_no_db['FimProcesso'] = datetime.now()
            tempo_de_processo = self._tempo_de_processo(processo_no_db['InicioProcesso'], processo_no_db['FimProcesso'])
            processo_no_db['TempoDeProcesso'] = tempo_de_processo

            await self.__db.tempo_processos.update_one(
                {
                    "SolicitacaoId": processo.SolicitacaoId,
                    "Fase": processo.Fase,
                    "DataInicio": processo_dict["DataInicio"],
                    "DataFim": processo_dict["DataFim"],
                },
                {"$set": processo_no_db
            })
            return True
        except Exception as e:
            print(f"❌ Erro ao atualizar o processo: {e}")
            return False

    async def buscar_por_solicitacao_id(self, solicitacao_id: int) -> list[ProcessoDto]:
        try:
            processos = await self.__db.tempo_processos.find(
                {
                    "SolicitacaoId": solicitacao_id
                }
            ).to_list(length=None)
            processos_dto = []

            for processo in processos:
                processo_dto = ProcessoDto(**processo)
                processos_dto.append(processo_dto)

            return processos_dto
        except Exception as e:
            print(f"❌ Erro ao buscar os processos por solicitação: {e}")
            return []
        
    def _tempo_de_processo(self, tempo_inicio: datetime, tempo_fim: datetime) -> str | None:
        if tempo_inicio and tempo_fim:
            delta = tempo_fim - tempo_inicio
            total_segundos = int(delta.total_seconds())

            horas = total_segundos // 3600
            minutos = (total_segundos % 3600) // 60
            segundos = total_segundos % 60

            tempo_formatado = f"{horas:02d}:{minutos:02d}:{segundos:02d}"
            tempo_formatado = "00:00:01" if tempo_formatado == "00:00:00" else tempo_formatado
            return tempo_formatado

        return None



from ast import Dict

from pydantic import BaseModel, Field


class ArquivoDTO(BaseModel):
    SolicitacaoId: int = Field(..., description="ID da solicitação")
    Cpf: str = Field(..., description="CPF do contribuinte")
    S1200: list[Dict] | None = Field(None, description="Lista de arquivos S-1200")
    S1210: list[Dict] | None = Field(None, description="Lista de arquivos S-1210")
    S2200: list[Dict] | None = Field(None, description="Lista de arquivos S-2200")
    S2299: list[Dict] | None = Field(None, description="Lista de arquivos S-2299")
    IdeEmpregador: Dict = Field(..., description="Identificação do empregador")
    Cnpj: str = Field(..., description="CNPJ do empregador")
    DataInicio: str = Field(..., description="Data de início no formato YYYY-MM-DD")
    DataFim: str = Field(..., description="Data de fim no formato YYYY-MM-DD")
    CertificadoId: int = Field(..., description="ID do certificado")
    DataProcessamento: str = Field(
        ..., description="Data de processamento no formato YYYY-MM-DD"
    )



from pymongo.errors import BulkWriteError

class CertificadoRepository:
    def __init__(self, db):
        self.__db = db

    async def inserir_certificado(self, certificado: dict) -> bool:
        try:
            certificado_no_db = await self.__db.certificado.find_one(
                {"cnpj": certificado["cnpj"]}
            )

            if certificado_no_db is None:
                await self.__db.certificado.insert_one(certificado)
                return True

            await self.__db.certificado.delete_one(
                {"cnpj": certificado["cnpj"]}
            )
            await self.__db.certificado.insert_one(certificado)
            return True
        except Exception as e:
            print(f"❌ Erro ao inserir o arquivo: {e}")
            return False

    async def remover_certificado(self, cnpj: str) -> bool:
        try:
            await self.__db.certificado.delete_many({"cnpj": cnpj})
            return True
        except Exception as e:
            print(f"❌ Erro ao remover o certificado: {e}")
            return False

    async def buscar_certificado_por_cnpj(self, cnpj: str) -> list[dict]:
        try:
            return await self.__db.certificado.find({"cnpj": cnpj}).to_list(length=None)
        except Exception as e:
            print(f"❌ Erro ao buscar certificado: {e}")
            return []


import uuid

class SolicitacaoXmlsRepository:
    def __init__(self, db):
        self.__db = db

    async def inserir_solicitacao_xml(self, solicitacao_xml: dict) -> bool:
        
        try:
            solicitacao_xml_no_db = await self.__db.solicitacao_xmls.find_one(
                {"solicitacaoId": solicitacao_xml["solicitacaoId"]}
            )

            if solicitacao_xml_no_db is None:
                await self.__db.solicitacao_xmls.insert_one(solicitacao_xml)
                return True

            await self.__db.solicitacao_xmls.delete_one(
                   {"solicitacaoId": solicitacao_xml["solicitacaoId"]}
            )
            await self.__db.solicitacao_xmls.insert_one(solicitacao_xml)
            return True
        except Exception as e:
            print(f"❌ Erro ao inserir o solicitacao xml: {e}")
            return False
        
    async def buscar_por_solicitacao_id(self, solicitacaoId: int) -> dict:
        try:
            return await self.__db.solicitacao_xmls.find_one({"solicitacaoId": solicitacaoId})
        except Exception as e:
            print(f"❌ Erro ao buscar solicitacao xml por solicitacaoId: {e}")
            return {}
        
    async def remover_por_solicitacao_id(self, solicitacaoId: int) -> bool:
        try:
            await self.__db.solicitacao_xmls.delete_one({"solicitacaoId": solicitacaoId})
            return True
        except Exception as e:
            print(f"❌ Erro ao remover solicitacao xml por solicitacaoId: {e}")
            return False


from cryptography.fernet import Fernet

class CryptoPassService:
    def __init__(self, key: bytes | None = None):
        # Se a chave não for passada, gera uma nova
        self.key = key or Fernet.generate_key()
        self.cipher = Fernet(self.key)

    def encrypt_password(self, password: str) -> bytes:
        # Recebe a senha em texto e retorna a senha criptografada em bytes
        password_bytes = password.encode()
        encrypted = self.cipher.encrypt(password_bytes)
        return encrypted

    def decrypt_password(self, token: bytes) -> str:
        # Recebe o token criptografado e retorna a senha em texto
        decrypted_bytes = self.cipher.decrypt(token)
        return decrypted_bytes.decode()


from datetime import date, datetime, timedelta
from pydantic import BaseModel, ConfigDict, Field, field_validator
from libretificacaotjcore.enums.e_fase_retificacao import EFaseRetificacao


class ProcessoDto(BaseModel):
    model_config = ConfigDict(frozen=False)
    SolicitacaoId: int = Field(..., description="ID da solicitação")
    DataInicio: date = Field(..., description="Data de início no formato YYYY-MM-DD")
    DataFim: date = Field(..., description="Data de fim no formato YYYY-MM-DD")
    Fase: int | None = Field(None, description="Fase de retificação")
    InicioProcesso: datetime | None = Field(None, description="Data e hora de início do processo")
    FimProcesso: datetime | None = Field(None, description="Data e hora de fim do processo")
    TempoDeProcesso: str | None = Field(None, description="Tempo de processamento")
    Observacoes: str | None = Field(None, description="Observações adicionais")

    # --- Validadores ---

    @field_validator("SolicitacaoId")
    @classmethod
    def validar_solicitacao_id(cls, v: int) -> int:
        if v <= 0:
            raise ValueError("O solicitacaoId deve ser um inteiro positivo.")
        return v

    @field_validator("DataInicio", mode="before")
    @classmethod
    def formatar_data_inicio(cls, value) -> date:
        """
        Aceita date, datetime ou string.
        """
        if isinstance(value, date):
            return value
        if isinstance(value, datetime):
            return value.date()
        if isinstance(value, str):
            try:
                return datetime.strptime(value, "%Y-%m-%d").date()
            except ValueError as ve:
                raise ValueError("A DataInicio deve estar no formato YYYY-MM-DD.") from ve
        raise TypeError("Tipo inválido para DataInicio.")

    @field_validator("DataFim", mode="before")
    @classmethod
    def ajustar_data_fim(cls, value) -> date:
        """
        Aceita date, datetime ou string.
        Retorna o último dia do mês da data fornecida.
        """
        if isinstance(value, datetime):
            value = value.date()
        if isinstance(value, date):
            ano, mes = value.year, value.month
        elif isinstance(value, str):
            try:
                ano, mes = map(int, value.split("-")[:2])
            except Exception:
                raise ValueError("A DataFim deve estar no formato YYYY-MM-DD.")
        else:
            raise TypeError("Tipo inválido para DataFim.")

        if mes < 1 or mes > 12:
            raise ValueError("A DataFim deve conter um mês válido (1–12).")

        # calcula o último dia do mês
        if mes == 12:
            proximo_mes = datetime(ano + 1, 1, 1)
        else:
            proximo_mes = datetime(ano, mes + 1, 1)
        return (proximo_mes - timedelta(days=1)).date()

    @field_validator("Fase")
    @classmethod
    def validar_fase(cls, v):
        if v is None:
            return v
        if v not in EFaseRetificacao:
            raise ValueError(f"Fase '{v}' não é válida.")
        return v



import httpx


class RequestServicoApi:
    def __init__(self, url, token):
        self.url = url
        self.token = token
        self.client = httpx.AsyncClient(timeout=120, verify=False)

    async def handler(self, *, mensagem_atualizacao: dict):
        print(self.token)
        response = await self.client.post(self.url, json=mensagem_atualizacao)

        if response.status_code != 200:
            raise Exception(f"Erro ao fazer request ao servico de API: {response.status_code}")
        
        await self.close()

    async def close(self):
        await self.client.aclose()